@@ -45,7 +45,6 @@
 #include "csm-session-fill.h"
 #include "csm-store.h"
 #include "csm-system.h"
-#include "csm-xsmp-server.h"
 #include "csm-fail-whale-dialog.h"
 
 #define CSM_DBUS_NAME "org.gnome.SessionManager"
@@ -284,7 +283,6 @@ main (int argc, char **argv)
         char             *display_str;
         CsmManager       *manager;
         CsmStore         *client_store;
-        CsmXsmpServer    *xsmp_server;
         MdmSignalHandler *signal_handler;
         static char     **override_autostart_dirs = NULL;
         static char      *session_name = NULL;
@@ -308,6 +306,12 @@ main (int argc, char **argv)
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
 
+        GSettings *settings = g_settings_new ("org.cinnamon.SessionManager");
+        if (g_settings_get_boolean (settings, "debug")) {
+            debug = TRUE;
+        }
+        g_object_unref(settings);
+
         sa.sa_handler = SIG_IGN;
         sa.sa_flags = 0;
         sigemptyset (&sa.sa_mask);
@@ -344,12 +348,43 @@ main (int argc, char **argv)
         csm_util_setenv ("DISPLAY", display_str);
         g_free (display_str);
 
+        const gchar *gtk_modules;
+        gchar *new_gtk_modules = NULL;
+
+        gtk_modules = g_getenv ("GTK_MODULES");
+
+        if (gtk_modules != NULL && g_strstr_len (gtk_modules, -1, "overlay-scrollbar")) {
+            int i = 0;
+            new_gtk_modules = g_strconcat ("", NULL);
+
+            gchar **module_list = g_strsplit (gtk_modules, ":", -1);
+
+            for (i = 0; i < g_strv_length (module_list); i++) {
+                if (!g_strstr_len (module_list[i], -1, "overlay-scrollbar")) {
+                    gchar *tmp = new_gtk_modules;
+                    new_gtk_modules = g_strconcat (tmp, ":", module_list[i], NULL);
+                    g_free (tmp);
+                }
+            }
+
+            g_strfreev (module_list);
+        }
+
+        if (new_gtk_modules) {
+            csm_util_setenv ("GTK_MODULES", new_gtk_modules);
+        }
+
+        g_free (new_gtk_modules);
+
         /* Some third-party programs rely on GNOME_DESKTOP_SESSION_ID to
          * detect if GNOME is running. We keep this for compatibility reasons.
          */
         csm_util_setenv ("GNOME_DESKTOP_SESSION_ID", "this-is-deprecated");
         csm_util_setenv ("CLUTTER_BACKEND", "x11");
 
+        /* Make QT5 apps follow the GTK style */
+        csm_util_setenv ("QT_STYLE_OVERRIDE", "gtk");
+
         client_store = csm_store_new ();
 
         /* Talk to logind before acquiring a name, since it does synchronous
@@ -359,8 +394,6 @@ main (int argc, char **argv)
          */
         g_object_unref (csm_get_system ());
 
-        xsmp_server = csm_xsmp_server_new (client_store);
-
         if (!acquire_name ()) {
                 csm_fail_whale_dialog_we_failed (TRUE, TRUE);
                 gtk_main ();
@@ -387,15 +420,10 @@ main (int argc, char **argv)
                 csm_util_init_error (TRUE, "Failed to load session \"%s\"", session_name ? session_name : "(null)");
         }
 
-        csm_xsmp_server_start (xsmp_server);
         csm_manager_start (manager);
 
         gtk_main ();
 
-        if (xsmp_server != NULL) {
-                g_object_unref (xsmp_server);
-        }
-
         if (manager != NULL) {
                 g_debug ("Unreffing manager");
                 g_object_unref (manager);
